package example;

/**
 * Класс примера работы с Меню
 */

import labir.utils.StatusbarManager;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;

import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.FormAttachment;

import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Composite;

public class MenuExample
{
	private static Display    display   ;
	private static Shell      shell     ;
	private        Composite  composite ;
	private        Tree       tree      ;
	private StatusbarManager  statusbar ;
	//--------------------------------------------------------------------------------
	/*
	 * Процедура создания текстового меню с подсветкой выделяемого пункта 
	 */
	private void createMenuBar ()
	{
		Menu menuBar = new Menu (shell, SWT.BAR);
		shell.setMenuBar (menuBar);
		
		MenuItem fileItem = new MenuItem (menuBar, SWT.CASCADE);
		fileItem.setText ("&File");
		
		Menu submenu = new Menu (shell, SWT.DROP_DOWN);
		fileItem.setMenu (submenu);

		MenuItem item = new MenuItem (submenu, SWT.PUSH);
		item.setText ("Select &All\tCtrl+A");
		item.setAccelerator (SWT.MOD1 + 'A');
		item.addListener (SWT.Selection, new Listener () {
			@Override
			public void handleEvent (Event e) {
				System.out.println ("Select All");
			}
		});
		MenuItem item2 = new MenuItem (submenu, SWT.PUSH);
		item2.setText ("Exit\tCtrl+Q");
		item2.setAccelerator (SWT.MOD1 + 'Q');
		item2.addListener (SWT.Selection, new Listener () {
			@Override
			public void handleEvent (Event e) {
				System.exit(0);
			}
		});
		
	    Listener armListener = new Listener() {
	        @Override
			public void handleEvent(Event event) {
	            MenuItem item = (MenuItem) event.widget;
	            statusbar.setText(0, item.getText());
	        }
	    };
	    Listener showListener = new Listener() {
	        @Override
			public void handleEvent(Event event) {
	            Menu menu = (Menu) event.widget;
	            MenuItem item = menu.getParentItem();
	            if (item != null)
		            statusbar.setText(0, item.getText());
	        }
	    };
	    Listener hideListener = new Listener() {
	        @Override
			public void handleEvent(Event event) {
	            statusbar.setText(0, "");
	        }
	    };

	    MenuItem editItem = new MenuItem(menuBar, SWT.CASCADE);
	    editItem.setText("Edit");
	    editItem.addListener(SWT.Arm, armListener);

	    Menu editMenu = new Menu(shell, SWT.DROP_DOWN);
	    editMenu.addListener(SWT.Hide, hideListener);
	    editMenu.addListener(SWT.Show, showListener);
	    String[] editStrings = { "Cut", "Copy", "Paste" };
	    editItem.setMenu(editMenu);
	    for (int i = 0; i < editStrings.length; i++) {
	        item = new MenuItem(editMenu, SWT.PUSH);
	        item.setText(editStrings[i]);
	        item.addListener(SWT.Arm, armListener);
	    }
	}
	//--------------------------------------------------------------------------------
	/*
	 * Процедура создания всплывающего меню 
	 */
	private void createPopupMenu()
	{
		// Создание контекстного меню
		Menu menu = new Menu (shell, SWT.POP_UP);
		MenuItem item1 = new MenuItem (menu, SWT.PUSH);
		item1.setText ("Color : yellow");
		item1.addListener (SWT.Selection, new Listener () {
			@Override
			public void handleEvent (Event e) {
				composite.setBackground(new Color (display,  255,  255,  64));
			}
		});

		Menu subMenu2 = new Menu (menu);
		MenuItem item2 = new MenuItem (menu, SWT.CASCADE);
		item2.setText ("Cascade Item");
		item2.setMenu (subMenu2);

		MenuItem subItem21 = new MenuItem (subMenu2, SWT.PUSH);
		MenuItem subItem22 = new MenuItem (subMenu2, SWT.PUSH);
		subItem21.setText ("Color : red");
		subItem22.setText ("Color : blue");

		subItem21.addListener (SWT.Selection, new Listener () {
			@Override
			public void handleEvent (Event e) {
				composite.setBackground(new Color (display,  255,  32,  32));
			}
		});

		subItem22.addListener (SWT.Selection, new Listener () {
			@Override
			public void handleEvent (Event e) {
				composite.setBackground(new Color (display,  32,  32,  255));
			}
		});
		// Создание вложенных подпунктов меню
		Menu subMenu3 = new Menu (menu);
		MenuItem item3 = new MenuItem (menu, SWT.CASCADE);
		item3.setText ("Radio Item");
		item3.setMenu (subMenu3);
		// Вложенные подпункты меню в виде "радиокнопок"
		MenuItem subItem31 = new MenuItem (subMenu3, SWT.RADIO);
		MenuItem subItem32 = new MenuItem (subMenu3, SWT.RADIO);
		subItem31.setText ("Color : magenta");
		subItem32.setText ("Color : cyan");

		subItem31.addListener (SWT.Selection, new Listener () {
			@Override
			public void handleEvent (Event e) {
				composite.setBackground(new Color (display,  255,  32,  255));
			}
		});

		subItem32.addListener (SWT.Selection, new Listener () {
			@Override
			public void handleEvent (Event e) {
				composite.setBackground(new Color (display,  32,  255,  255));
			}
		});
		// Подключение контекстного меню к панели
		composite.setMenu (menu);
	}
	//--------------------------------------------------------------------------------
	/*
	 * Динамически блокируемые пункты контекстного меню 
	 */
	private void createPopupDynamic()
	{
		shell.setLayout(new FormLayout());

        FormData fd = new FormData(SWT.DEFAULT, SWT.DEFAULT);
        fd.top    = new FormAttachment(0, 10);
        fd.left   = new FormAttachment(0, 10);
        fd.width  = 100;
        fd.height = 180;
		
		composite = new Composite (shell, SWT.BORDER);
		composite.setLayoutData(fd);
		composite.setBackground(new Color (shell.getDisplay(), 255, 16, 255));
		
        statusbar = new StatusbarManager(shell);
		statusbar.setItems(new String[][]{{"left", "-1", "label"}});

        fd = new FormData(SWT.DEFAULT, SWT.DEFAULT);
        fd.left   = new FormAttachment(  0, 0 );
        fd.right  = new FormAttachment(100, 0 );
        fd.bottom = new FormAttachment(100, 0);
        fd.height = statusbar.getHeight();
        
        statusbar.setLayoutData(fd);
        
		// создать визуальный компонент дерева
		tree = new Tree (shell, SWT.BORDER | SWT.MULTI);
        fd = new FormData(SWT.DEFAULT, SWT.DEFAULT);
        fd.top    = new FormAttachment(  0, 10 );
        fd.left   = new FormAttachment(  0, 300);
        fd.width  = 100;
        fd.height = 160;
   		tree.setLayoutData(fd);
		
		// контекстное меню создать
		final Menu menu = new Menu (shell, SWT.POP_UP);
		// подключить контекстное меню
		tree.setMenu (menu);
		for (int i = 0; i < 9; i++) {
			TreeItem treeItem = new TreeItem (tree, SWT.NONE);
			treeItem.setText ("Item " + i);
			MenuItem menuItem = new MenuItem (menu, SWT.PUSH);
			menuItem.setText (treeItem.getText ());
		}
		// обработчик события открытия контекстного меню
		menu.addListener (SWT.Show, new Listener () {
			@Override
			public void handleEvent (Event event) {
				MenuItem [] menuItems = menu.getItems ();
				TreeItem [] treeItems = tree.getSelection ();
				for (int i=0; i < menuItems.length; i++) {
					String text = menuItems [i].getText ();
					int index = 0;
					while (index < treeItems.length) {
						if (treeItems [index].getText ().equals (text))
							break;
						index++;
					}
					menuItems [i].setEnabled (index != treeItems.length);
				}
			}
		});
	}
	//--------------------------------------------------------------------------------
	public static void main(String[] args)
	{
		display = new Display ();
		shell = new Shell (display);

		shell.setSize (500, 350);
		shell.setText("Menu examples");
		
		MenuExample em = new MenuExample();
		// Создание основного меню
		em.createMenuBar     ();
		// Создание всплывающего меню с динамическим блокированием пункта меню
		em.createPopupDynamic();
		// Создание всплывающего меню
		em.createPopupMenu   ();
		// Подключение к shell всплывающего меню
		shell.addListener (SWT.MenuDetect, new Listener () {
			@Override
			public void handleEvent (Event event) {
				Menu menu = new Menu (shell, SWT.POP_UP);
				MenuItem item = new MenuItem (menu, SWT.PUSH);
				item.setText ("Menu Item (shell)");
				item.addListener (SWT.Selection, new Listener () {
					@Override
					public void handleEvent (Event e) {
						System.out.println ("Popup menu of shell is selected");
					}
				});
				// Позиционирование всплывающего окна
				menu.setLocation (event.x, event.y);
				menu.setVisible (true);
				while (!menu.isDisposed () && menu.isVisible ()) {
					if (!display.readAndDispatch ()) 
						display.sleep ();
				}
				/*
				 * next operator needed, to get the selection event,
				 * which is fired AFTER the menu is hidden 
				 */
				
				while (display.readAndDispatch());
				menu.dispose ();
			}
		});

		shell.open ();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch ()) display.sleep ();
		}
		display.dispose ();
	}
}
